import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Dictionary object contains objects as in a map
 * but ordered as a List, ie. put at the end of the list.
 * But keys need to be unique.
 * 
 * @author julien - InWebDesign
 * Created on 30 juil. 2003
 */
public class Dictionary {
	/** key / index association map */
	private Map keysIndexDic;
	/** index / key association map */
	private Map indexKeysDic;
	/** Values of the dictionary */
	private List valuesDic;
	
	/**
	 * Constructor for a Dictionary object.
	 */
	public Dictionary() {
		super();
		keysIndexDic = new HashMap();
		indexKeysDic = new HashMap();
		valuesDic = new ArrayList();
	}
		
	/**
	 * Get an object with the given index.
	 * @param index Position of the object.
	 * @return The corresponding object
	 */
	public synchronized Object get(int index){
        if(valuesDic.size() -1 < index || index < 0)
            return null;
		return valuesDic.get(index);	
	}
	
	/**
	 * Get an object with the given key.
	 * @param key The key which corresponds to the object.
	 * @return the object
	 */
	public synchronized Object get(Object key){
		Integer intIndex = (Integer)keysIndexDic.get(key);
		if(intIndex != null)
			return valuesDic.get(intIndex.intValue());
		else
			return null;
	}
	
	/**
	 * Return the index of the object defined by the key.
	 * @param key
	 * @return
	 */
	public synchronized int getIndex(Object key){
	    Integer intIndex = (Integer)keysIndexDic.get(key);
		if(intIndex != null)
		    return intIndex.intValue();
		else
		    return -1;
	}
	
	/**
	 * Get the key corresponding to the index.
	 * @param index Index of the object
	 * @return the object
	 */
	public synchronized Object getKey(int index){
		Object obj = indexKeysDic.get(new Integer(index));
		return obj;
	}
	
	/**
	 * Return if the key is in the dictionary.
	 * @param key The key to find out
	 * @return if the key is found
	 */
	public synchronized boolean containsKey(Object key){
		return keysIndexDic.containsKey(key);	
	}
	
	/**
	 * Return if the value is in the dictionnary.
	 * @param value The object 
	 * @return if the value is in the dictionnary
	 */
	public synchronized boolean containsValue(Object value){
		return valuesDic.contains(value);	
	}
	
	/**
	 * Return the size of the dictionary.
	 * @return The size
	 */
	public synchronized int size(){
		return valuesDic.size();	
	}
	
	/**
	 * Retrun the key set of the dictionary.
	 * @return the set of the dictionnary
	 */
	public synchronized Set keySet(){
		return keysIndexDic.keySet();	
	}
	
	/**
	 * Put an object at the end of the dictionary.
	 * @param key The key
	 * @param value The object to store
	 */
	public synchronized void put(Object key, Object value){
        int index = valuesDic.size();
        Integer intIndex = (Integer)keysIndexDic.get(key);
        if(intIndex != null)
            index = intIndex.intValue();
        
		keysIndexDic.put(key, new Integer(index));
		indexKeysDic.put(new Integer(index),key);
        
        if(intIndex != null){
        	valuesDic.remove(index);
            valuesDic.add(index,value);        	
        }else{
        	valuesDic.add(value);
        }
	}
	
	/**
	 * Remove the object linked to this key from the dictionnary.
	 * @param key
	 */
	public synchronized void remove(Object key){
        Integer intIndex = (Integer)keysIndexDic.get(key);
        if(intIndex != null){
            valuesDic.remove(intIndex.intValue());
            keysIndexDic.remove(key);
            indexKeysDic.remove(intIndex);
            // Move all elements to the left
            for(int i = intIndex.intValue()+1; i < valuesDic.size() + 1; i++){
                Object aKey = indexKeysDic.get(new Integer(i));
                keysIndexDic.put(aKey, new Integer(i - 1));
        		indexKeysDic.put(new Integer(i - 1),aKey);
            }
        }
	}
	
	/**
	 * Remove the object at the given index from the dictionnary.
	 * @param index
	 */
	public synchronized void remove(int index){
        Object o = getKey(index);
        if(o != null)
            remove(o);
        else
            throw new ArrayIndexOutOfBoundsException("Object not found at index : "+index);
	}
	
	/**
	 * Clear the dictionary.
	 */
	public synchronized void clear(){
		indexKeysDic.clear();
		keysIndexDic.clear();
		valuesDic.clear();
	}
	
	/**
	 * Put another dictionary at the end of this one
	 * @param d the dictionnary to add
	 */
	public synchronized void putAll(Dictionary d){
		for(int i = 0; i < d.size(); i++){
			Object key = d.getKey(i);
			Object value = d.get(i);
			this.put(key, value);	
		}
	}
	
	
}
